/*******************************************************
**** ROUTINES FOR FAT32 IMPLEMATATION OF SD CARD ****
********************************************************
Controller: ATmega32
Compiler	: AVR-GCC
Version	: 2.2
Author	: CC Dharmani, Chennai (India)
www.dharmanitech.com
Date		: 15 July 2009
********************************************************/

/*======================================================
modifie par Silicium628 (France) 
Un grand merci à CC Dharmani pour l'ensemble de son travail!
1 fevrier 2010
http://silicium628.chez-alice.fr/
http://www.silicium628.fr/

-suppression de toutes les fonctions d'écriture
-ajout des fonctions: 
	-read_block_in_File
	-read_string_in_File
========================================================*/

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "FAT32_628.h"
#include "dm_lcd.h"
#include "SD_routines_628.h"


/****************************************************************************
Function: to read data from boot sector of SD card, to determine important
parameters like bytesPerSector, sectorsPerCluster etc.
Arguments: none
return: none
****************************************************************************/
uint8_t getBootSectorData (void)
{
struct BS_Structure *bpb; //mapping the buffer onto the structure
struct MBRinfo_Structure *mbr;
struct partitionInfo_Structure *partition;
uint32_t dataSectors;

unusedSectors = 0;

SD_readSingleBlock(0);
bpb = (struct BS_Structure *)buffer;

if(bpb->jumpBoot[0]!=0xE9 && bpb->jumpBoot[0]!=0xEB)   //check if it is boot sector
{
  mbr = (struct MBRinfo_Structure *) buffer;       //if it is not boot sector, it must be MBR
  
  if(mbr->signature != 0xaa55) return 1;       //if it is not even MBR then it's not FAT32
  	
  partition = (struct partitionInfo_Structure *)(mbr->partitionData);//first partition
  unusedSectors = partition->firstSector; //the unused sectors, hidden to the FAT
  
  SD_readSingleBlock(partition->firstSector);//read the bpb sector
  bpb = (struct BS_Structure *)buffer;
  if(bpb->jumpBoot[0]!=0xE9 && bpb->jumpBoot[0]!=0xEB) return 1; 
}

bytesPerSector = bpb->bytesPerSector;
sectorPerCluster = bpb->sectorPerCluster;
reservedSectorCount = bpb->reservedSectorCount;
rootCluster = bpb->rootCluster;// + (sector / sectorPerCluster) +1;
firstDataSector = bpb->hiddenSectors + reservedSectorCount + (bpb->numberofFATs * bpb->FATsize_F32);

dataSectors = bpb->totalSectors_F32
              - bpb->reservedSectorCount
              - ( bpb->numberofFATs * bpb->FATsize_F32);
totalClusters = dataSectors / sectorPerCluster;


/*
if((getSetFreeCluster (TOTAL_FREE, GET, 0)) > totalClusters)  //check if FSinfo free clusters count is valid
     freeClusterCountUpdated = 0;
else
	 freeClusterCountUpdated = 1;
	*/ 

	 
return 0;
}

/***************************************************************************
Function: to calculate first sector address of any given cluster
Arguments: cluster number for which first sector is to be found
return: first sector address
***************************************************************************/
uint32_t getFirstSector(uint32_t clusterNumber)
{
  return (((clusterNumber - 2) * sectorPerCluster) + firstDataSector);
}

/***************************************************************************
Function: get cluster entry value from FAT to find out the next cluster in the chain
or set new cluster entry in FAT
Arguments: 1. current cluster number, 2. get_set (=GET, if next cluster is to be found or = SET,
if next cluster is to be set 3. next cluster number, if argument#2 = SET, else 0
return: next cluster number, if if argument#2 = GET, else 0
****************************************************************************/
uint32_t getSetNextCluster (uint32_t clusterNumber,
                                 uint8_t get_set,
                                 uint32_t clusterEntry)
{
uint16_t FATEntryOffset;
uint32_t *FATEntryValue;
uint32_t FATEntrySector;
uint8_t retry = 0;

//get sector number of the cluster entry in the FAT
FATEntrySector = unusedSectors + reservedSectorCount + ((clusterNumber * 4) / bytesPerSector) ;

//get the offset address in that sector number
FATEntryOffset = (uint16_t) ((clusterNumber * 4) % bytesPerSector);

//read the sector into a buffer
while(retry <10)
{ if(!SD_readSingleBlock(FATEntrySector)) break; retry++;}

//get the cluster address from the buffer
FATEntryValue = (uint32_t *) &buffer[FATEntryOffset];

if(get_set == GET)  return ((*FATEntryValue) & 0x0fffffff);


*FATEntryValue = clusterEntry;   //for setting new value in cluster entry in FAT

// SD_writeSingleBlock(FATEntrySector);

return (0);
}


/***************************************************************************
Function: to get DIR/FILE list or a single file address (cluster number) or to delete a specified file
Arguments: #1 - flag: GET_LIST, GET_FILE or DELETE #2 - pointer to file name (0 if arg#1 is GET_LIST)
return: first cluster of the file, if flag = GET_FILE
        print file/dir list of the root directory, if flag = GET_LIST
		  Delete the file mentioned in arg#2, if flag = DELETE
****************************************************************************/
struct dir_Structure* findFiles (uint8_t flag, uint8_t *fileName)
{
uint32_t cluster, sector, firstSector;
//, firstCluster, nextCluster;
struct dir_Structure *dir;
uint16_t i;
uint8_t j;

cluster = rootCluster; //root cluster

while(1)
{
   firstSector = getFirstSector (cluster);

   for(sector = 0; sector < sectorPerCluster; sector++)
   {
     SD_readSingleBlock (firstSector + sector);
	

     for(i=0; i<bytesPerSector; i+=32)
     {
	    dir = (struct dir_Structure *) &buffer[i];

		if(dir->name[0] == EMPTY) //indicates end of the file list of the directory
		{
			if((flag == GET_FILE) || (flag == DELETE)) 
			{ 
				lcd_puts("File does not exist!");
				_delay_ms(1000);
			}
			return 0;   
		}
		if((dir->name[0] != DELETED) && (dir->attrib != ATTR_LONG_NAME))
        {
          if((flag == GET_FILE) || (flag == DELETE))
          {
            for(j=0; j<11; j++)
            if(dir->name[j] != fileName[j]) break;
            if(j == 11)
			{
			  if(flag == GET_FILE)
              {
			    appendFileSector = firstSector + sector;
				appendFileLocation = i;
				appendStartCluster = (((uint32_t) dir->firstClusterHI) << 16) | dir->firstClusterLO;
				fileSize = dir->fileSize;
			    return (dir);
			  }	
            }
          }
          else  //when flag = GET_LIST
		  {
		     // TX_NEWLINE;
			  for(j=0; j<11; j++)
			  {
			   // if(j == 8) transmitByte(' ');
			   // transmitByte (dir->name[j]);
			  }
		     // transmitString_F (PSTR("   "));
			  if((dir->attrib != 0x10) && (dir->attrib != 0x08))
			  {
			     // transmitString_F (PSTR("FILE" ));
		         // transmitString_F (PSTR("   "));
			     // displayMemory (dir->fileSize);
			  }
			  else
			   // transmitString_F ((dir->attrib == 0x10)? PSTR("DIR") : PSTR("ROOT"))
				  ;
		  }
		}
	 }
   }

   cluster = (getSetNextCluster (cluster, GET, 0));

   if(cluster > 0x0ffffff6) return 0;
   if(cluster == 0) 
   { 
	   lcd_puts("Err getting cluster"); 
	   _delay_ms(1000); 
	   return 0;
   }
}
return 0;
}

/***************************************************************************
Function: if flag=READ then to read file from SD card
if flag=VERIFY then functions will verify whether a specified file is already existing
Arguments: flag (READ or VERIFY) and pointer to the file name
return: 0, if normal operation or flag is READ
	      1, if file is already existing and flag = VERIFY
		  2, if file name is incompatible
***************************************************************************/
uint8_t readFile (uint8_t flag, uint8_t *fileName)
{
struct dir_Structure *dir;
uint32_t cluster, byteCounter = 0, fileSize, firstSector;
uint16_t k;
uint8_t j, error;

error = convertFileName (fileName); //convert fileName into FAT format
if(error) return 2;

dir = findFiles (GET_FILE, fileName); //get the file location
if(dir == 0) 
{
	lcd_clrscr();
	lcd_puts("dir = 0"); 
	_delay_ms(1000);
	return (0);
	lcd_clrscr();
}	

if(flag == VERIFY) return (1);	//specified file name is already existing

cluster = (((uint32_t) dir->firstClusterHI) << 16) | dir->firstClusterLO;

fileSize = dir->fileSize;

while(1)
{
	firstSector = getFirstSector (cluster);

	for(j=0; j<sectorPerCluster; j++)
	{
		
		SD_readSingleBlock(firstSector + j);
    
		for(k=0; k<512; k++)
		{
			lcd_putc(buffer[k]);
			_delay_ms(20);
			if ((byteCounter++) >= fileSize ) return 0;
		}
	}
	cluster = getSetNextCluster (cluster, GET, 0);
	if(cluster == 0) 
	{	
		lcd_puts("Error in getting cluster"); 
		_delay_ms(1000);
		return 0;
	}
}
return 0;
}

/* ***************************************************************************
lit un certain nombre d'octets contigus dans un fichier
Arguments: 
-index: adresse du début de la ligne à lire dans le fichier
-longueur du bloc à lire (nombre d'octets à lire)
-pointeur sur le nom de fichier
****************************************************************************/
uint8_t read_block_in_File ( uint16_t index, uint8_t longueur, uint8_t *fileName)
{
	struct dir_Structure *dir;
	uint32_t cluster, byteCounter = 0, fileSize, firstSector;
	uint16_t k;
	uint8_t j, error;
	
	error = convertFileName (fileName); //convert fileName into FAT format
	if(error) return 2;

	dir = findFiles (GET_FILE, fileName); //get the file location
	if(dir == 0) 
	{
		lcd_clrscr();
		lcd_puts("dir = 0"); 
		_delay_ms(1000);
		lcd_clrscr();
		return (0);
	}	
	cluster = (((uint32_t) dir->firstClusterHI) << 16) | dir->firstClusterLO;
	fileSize = dir->fileSize;
	byteCounter = 0;
	while(1)
	{
		firstSector = getFirstSector (cluster);
		for(j=0; j<sectorPerCluster; j++)
		{
			SD_readSingleBlock(firstSector + j);
			for(k=0; k<512; k++)
			{
				if (byteCounter >= index)
				{
					lcd_putc(buffer[k]);
					_delay_ms(20);
				}
				byteCounter++;
				if ((byteCounter > (index + longueur)) || (byteCounter >= fileSize )) return 0;
			}
		}
		cluster = getSetNextCluster (cluster, GET, 0);
		if(cluster == 0) 
		{	
			lcd_puts("Error in getting cluster"); 
			_delay_ms(1000);
			return 0;
		}
	}
	return 0;
}


/* ***************************************************************************
lit un certain nombre d'octets contigus (séquence) dans un fichier
avec comme critère de recherche le début de la séquence
Arguments: 
-pointeur sur l'étiquette (début de la séquence = 5 octets) 
-pointeur sur le nom de fichier dans la SD-card
Ecrit le résultat dans le str_lu (de type uint8_t[40] , déclaré dans SD_routines_628.h) 
****************************************************************************/
uint8_t read_string_in_File ( char *etiquette, char *fileName)
// attention: ETIQUETTE = 6 caracteres à modifier plus bas 	dans la boucle, le cas échéant
{
	struct dir_Structure *dir;
	uint32_t cluster, byteCounter = 0, fileSize, firstSector;
	uint16_t k;
	uint8_t i, j, ok, error;
	
	error = convertFileName (fileName); //convert fileName into FAT format
	if(error) return 2;

	dir = findFiles (GET_FILE, fileName); //get the file location
	if(dir == 0) 
	{
		lcd_clrscr();
		lcd_puts("dir = 0"); 
		_delay_ms(1000);
		lcd_clrscr();
		return (0);
	}	
	cluster = (((uint32_t) dir->firstClusterHI) << 16) | dir->firstClusterLO;
	fileSize = dir->fileSize;
	byteCounter = 0;
	while(1)
	{
		firstSector = getFirstSector (cluster);
		for(j=0; j<sectorPerCluster; j++)
		{
			SD_readSingleBlock(firstSector + j);
			for(k=0; k<512; k++)
			{
				//décallage à gauche du str_lu
				for (i=0; i<40; i++) {str_lu[i] = str_lu[i+1]; }
				str_lu[39]=buffer[k];
				//compare buffer[0..5] avec l'etiquette
				ok=1;
				for (i=0; i<6; i++) 
				{ 
					if (str_lu[i] != etiquette[i]) { ok=0; }	
				}
				if (ok) return (0);	//sortir si comparaison ok - la séquence lue est dans str_lu

				byteCounter++;
				lcd_gotoxy(10, 2);
				lcd_putc(65 + ((byteCounter / 2000 ) % 26)); // tit' animation... pour patienter
				if (byteCounter >= fileSize ) return 1;// (séquence pas trouvée)
			}
		}
		cluster = getSetNextCluster (cluster, GET, 0);
		if(cluster == 0) return 2; // (Error in getting cluster)
	}
	return 0;
}


//***************************************************************************
//Function: to convert normal short file name into FAT format
//Arguments: pointer to the file name
//return: 0, if successful else 1.
//***************************************************************************
uint8_t convertFileName (uint8_t *fileName)
{
uint8_t fileNameFAT[11];
uint8_t j, k;

for(j=0; j<12; j++)
if(fileName[j] == '.') break;

if(j>8) 
{   
	lcd_puts("Invalid fileName.."); 
	_delay_ms(1000);
	return 1;
}

for(k=0; k<j; k++) //setting file name
  fileNameFAT[k] = fileName[k];

for(k=j; k<=7; k++) //filling file name trail with blanks
  fileNameFAT[k] = ' ';

j++;
for(k=8; k<11; k++) //setting file extention
{
  if(fileName[j] != 0)
    fileNameFAT[k] = fileName[j++];
  else //filling extension trail with blanks
    while(k<11)
      fileNameFAT[k++] = ' ';
}

for(j=0; j<11; j++) //converting small letters to caps
  if((fileNameFAT[j] >= 0x61) && (fileNameFAT[j] <= 0x7a))
    fileNameFAT[j] -= 0x20;

for(j=0; j<11; j++)
  fileName[j] = fileNameFAT[j];

// 	lcd_puts("conv FileName ok"); 
// 	_delay_ms(1000);
return 0;
}



//***************************************************************************
//Function: to search for the next free cluster in the root directory
//          starting from a specified cluster
//Arguments: Starting cluster
//return: the next free cluster
//****************************************************************
uint32_t searchNextFreeCluster (uint32_t startCluster)
{
  uint32_t cluster, *value, sector;
  uint8_t i;
    
	startCluster -=  (startCluster % 128);   //to start with the first file in a FAT sector
    for(cluster =startCluster; cluster <totalClusters; cluster+=128) 
    {
      sector = unusedSectors + reservedSectorCount + ((cluster * 4) / bytesPerSector);
      SD_readSingleBlock(sector);
      for(i=0; i<128; i++)
      {
       	 value = (uint32_t *) &buffer[i*4];
         if(((*value) & 0x0fffffff) == 0)
            return(cluster+i);
      }  
    } 

 return 0;
}


