/*******************************************************
**** ROUTINES FOR FAT32 IMPLEMATATION OF SD CARD ****
********************************************************
Controller: ATmega32
Compiler	: AVR-GCC
Version	: 2.2
Author	: CC Dharmani, Chennai (India)
www.dharmanitech.com
Date		: 15 July 2009
********************************************************/

/*======================================================
modifie par Silicium628 (France) 
Un grand merci à CC Dharmani pour l'ensemble de son travail!
15 octobre 2009
http://silicium628.chez-alice.fr/
http://www.silicium628.fr/
-suppression de toutes fes fonctions d'écriture
-ajout des fonctions: 
	-read_block_in_File
	-read_string_in_File
========================================================*/

#ifndef _FAT32_H_
#define _FAT32_H_

//Structure to access Master Boot Record for getting info about partioions
struct MBRinfo_Structure{
uint8_t	nothing[446];		//ignore, placed here to fill the gap in the structure
uint8_t	partitionData[64];	//partition records (16x4)
uint16_t	signature;		//0xaa55
};

//Structure to access info of the first partition of the disk 
struct partitionInfo_Structure{ 				
uint8_t	status;				//0x80 - active partition
uint8_t 	headStart;			//starting head
uint16_t	cylSectStart;		//starting cylinder and sector
uint8_t	type;				//partition type 
uint8_t	headEnd;			//ending head of the partition
uint16_t	cylSectEnd;			//ending cylinder and sector
uint32_t	firstSector;		//total sectors between MBR & the first sector of the partition
uint32_t	sectorsTotal;		//size of this partition in sectors
};

//Structure to access boot sector data
struct BS_Structure{
uint8_t jumpBoot[3]; //default: 0x009000EB
uint8_t OEMName[8];
uint16_t bytesPerSector; //default: 512
uint8_t sectorPerCluster;
uint16_t reservedSectorCount;
uint8_t numberofFATs;
uint16_t rootEntryCount;
uint16_t totalSectors_F16; //must be 0 for FAT32
uint8_t mediaType;
uint16_t FATsize_F16; //must be 0 for FAT32
uint16_t sectorsPerTrack;
uint16_t numberofHeads;
uint32_t hiddenSectors;
uint32_t totalSectors_F32;
uint32_t FATsize_F32; //count of sectors occupied by one FAT
uint16_t extFlags;
uint16_t FSversion; //0x0000 (defines version 0.0)
uint32_t rootCluster; //first cluster of root directory (=2)
uint16_t FSinfo; //sector number of FSinfo structure (=1)
uint16_t BackupBootSector;
uint8_t reserved[12];
uint8_t driveNumber;
uint8_t reserved1;
uint8_t bootSignature;
uint32_t volumeID;
uint8_t volumeLabel[11]; //"NO NAME "
uint8_t fileSystemType[8]; //"FAT32"
uint8_t bootData[420];
uint16_t bootEndSignature; //0xaa55
};


//Structure to access FSinfo sector data
struct FSInfo_Structure
{
uint32_t leadSignature; //0x41615252
uint8_t reserved1[480];
uint32_t structureSignature; //0x61417272
uint32_t freeClusterCount; //initial: 0xffffffff
uint32_t nextFreeCluster; //initial: 0xffffffff
uint8_t reserved2[12];
uint32_t trailSignature; //0xaa550000
};

//Structure to access Directory Entry in the FAT
struct dir_Structure{
uint8_t name[11];
uint8_t attrib; //file attributes
uint8_t NTreserved; //always 0
uint8_t timeTenth; //tenths of seconds, set to 0 here
uint16_t createTime; //time file was created
uint16_t createDate; //date file was created
uint16_t lastAccessDate;
uint16_t firstClusterHI; //higher word of the first cluster number
uint16_t writeTime; //time of last write
uint16_t writeDate; //date of last write
uint16_t firstClusterLO; //lower word of the first cluster number
uint32_t fileSize; //size of file in bytes
};

//Attribute definitions for file/directory
#define ATTR_READ_ONLY     0x01
#define ATTR_HIDDEN        0x02
#define ATTR_SYSTEM        0x04
#define ATTR_VOLUME_ID     0x08
#define ATTR_DIRECTORY     0x10
#define ATTR_ARCHIVE       0x20
#define ATTR_LONG_NAME     0x0f


#define DIR_ENTRY_SIZE     0x32
#define EMPTY              0x00
#define DELETED            0xe5
#define GET     0
#define SET     1
#define READ	0
#define VERIFY  1
#define ADD		0
#define REMOVE	1
#define TOTAL_FREE   1
#define NEXT_FREE    2
#define GET_LIST     0
#define GET_FILE     1
#define DELETE		 2
#define EOF32		0x0fffffff


//************* external variables *************
volatile uint32_t firstDataSector, rootCluster, totalClusters;
volatile uint16_t  bytesPerSector, sectorPerCluster, reservedSectorCount;
uint32_t unusedSectors, appendFileSector, appendFileLocation, fileSize, appendStartCluster;

//global flag to keep track of free cluster count updating in FSinfo sector
uint8_t freeClusterCountUpdated;



//************* functions *************
uint8_t getBootSectorData (void);
uint32_t getFirstSector(uint32_t clusterNumber);
uint32_t getSetFreeCluster(uint8_t totOrNext, uint8_t get_set, uint32_t FSEntry);
struct dir_Structure* findFiles (uint8_t flag, uint8_t *fileName);
uint32_t getSetNextCluster (uint32_t clusterNumber,uint8_t get_set,uint32_t clusterEntry);
uint8_t readFile (uint8_t flag, uint8_t *fileName);
uint8_t read_string_in_File ( char *etiquette, char *fileName);  // (ajout Silicium628)
uint8_t convertFileName (uint8_t *fileName);
void appendFile (void);
uint32_t searchNextFreeCluster (uint32_t startCluster);


#endif
