/**************************************************************
****** FUNCTIONS FOR SD RAW DATA TRANSFER *******
**************************************************************
Controller: ATmega32 (8 Mhz internal)
Compiler	: AVR-GCC
Version	: 2.2
Author	: CC Dharmani, Chennai (India)
			  www.dharmanitech.com
Date		: 15 July 2009
**************************************************************/

/*======================================================
modifie par Silicium628 (France) 
Un grand merci à CC Dharmani pour l'ensemble de son travail!
15 octobre 2009
http://silicium628.chez-alice.fr/
http://www.silicium628.fr/
========================================================*/


#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "SPI_routines.h"
#include "SD_routines_628.h"
#include "dm_lcd.h"


// #include "UART_routines.h"

/******************************************************************
Function: to initialize the SD card in SPI mode
Arguments: none
return: uint8_t; will be 0 if no error,
otherwise the response byte will be sent
******************************************************************/
uint8_t SD_init(void)
{
uint8_t i, response, retry=0 ;

SD_CS_ASSERT; // macro définie dans SD_routines_628.h
do
{
   for(i=0; i<10; i++) {  SPI_transmit(0xff); }
   response = SD_sendCommand(GO_IDLE_STATE, 0); //send 'reset & go idle' command
   

   retry++;
   if(retry>0xfe) 
   {
	  lcd_clrscr();
	  lcd_puts("SD init fail..");
	  _delay_ms(500);
	  return 1;
   } //time out
   
} while(response != 0x01);

SD_CS_DEASSERT;
SPI_transmit (0xff);
SPI_transmit (0xff);

retry = 0;



do
{
    response = SD_sendCommand(SEND_OP_COND, 0); //activate card's initialization process
	response = SD_sendCommand(SEND_OP_COND, 0); //same command sent again for compatibility with some cards
    retry++;
    if(retry>0xfe) return 1; //time out
}while(response);

SD_sendCommand(CRC_ON_OFF, OFF); //disable CRC; default - CRC disabled in SPI mode
SD_sendCommand(SET_BLOCK_LEN, 512); //set block size to 512

return 0; //normal return
}

/******************************************************************
Function: to send a command to SD card
Arguments: uint8_t (8-bit command value)
& uint32_t (32-bit command argument)
return: uint8_t; response byte
******************************************************************/
uint8_t SD_sendCommand(uint8_t cmd, uint32_t arg)
{
uint8_t response, retry=0;

SD_CS_ASSERT;

SPI_transmit(cmd | 0x40); //send command, first two bits always '01'
SPI_transmit(arg>>24);
SPI_transmit(arg>>16);
SPI_transmit(arg>>8);
SPI_transmit(arg);
SPI_transmit(0x95);

while((response = SPI_receive()) == 0xff) //wait response
   if(retry++ > 0xfe) break; //time out error

SPI_receive(); //extra 8 CLK
SD_CS_DEASSERT;

return response; //return state
}


void lcd_aff_octet (uint8_t valeur1)
//affiche un nombre positif 0..255 en representation decimale
{
	uint8_t r ;
	char tbl[3+1];
	uint8_t i;

	for (i=1; i<=3; i++) 
	{
		r=48 + valeur1 % 10; // modulo (reste de la division)
		tbl[i]=r;
		valeur1 /= 10; // quotient
	}
	for (i=1; i<=3; i++) { lcd_putc(tbl[3+1-i]); } 
	
}

/******************************************************************
Function: to read a single block from SD card
Arguments: none
return: uint8_t; will be 0 if no error,
otherwise the response byte will be sent
******************************************************************/
uint8_t SD_readSingleBlock(uint32_t startBlock)
{
	uint8_t response;
	unsigned int i, retry=0;

	response = SD_sendCommand(READ_SINGLE_BLOCK, startBlock<<9); //read a Block command
//block address converted to starting address of 512 byte Block
	if(response != 0x00) //check for SD status: 0x00 - OK (No flags set)
		return response;

	SD_CS_ASSERT;

	retry = 0;
	while(SPI_receive() != 0xfe) //wait for start block token 0xfe (0x11111110)
		if(retry++ > 0xfffe){SD_CS_DEASSERT; return 1;} //return if time-out

	for(i=0; i<512; i++) //read 512 bytes
	{
		buffer[i] = SPI_receive();
	}
	SPI_receive(); //receive incoming CRC (16-bit), CRC is ignored here
	SPI_receive();

	SPI_receive(); //extra 8 clock pulses
	SD_CS_DEASSERT;
	
	return 0;
}

