/**************************************************************
FUNCTIONS FOR SPI COMMUNICATION 
**************************************************************/

#include <avr/io.h>
#include "SPI_routines.h"


//SPI initialize for SD card
//clock rate: 125Khz
// voir p:134
void spi_init(void)
{
/*	
SPCR:	• Bit 5 – DORD: Data Order
			When the DORD bit is written to one, the LSB of the data word is transmitted first.
			When the DORD bit is written to zero, the MSB of the data word is transmitted first.
*/
	
//setup SPI: Master mode, MSB first, SCK phase low, SCK idle low	
SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR1)|(1<<SPR0); //clock rate fck/128 = 16E6/128 = 125kHz
SPSR = 0x00; //Bit 0 = SPI2X: si=1 -> Double SPI Speed Bit voir p:138
}

unsigned char SPI_transmit(unsigned char data)
{
// Start transmission
SPDR = data;

// Wait for transmission complete
while(!(SPSR & (1<<SPIF)));
data = SPDR;

return(data);
}

unsigned char SPI_receive(void)
{
unsigned char data;
// Wait for reception complete

SPDR = 0xff;
while(!(SPSR & (1<<SPIF)));
data = SPDR;

// Return data register
return data;
}

